/********************************************************************
*h*
*   Name:	
*
*   		dp83820.c
*
*   Description :	
*
*   		This file contains entry point functions.
*  
* 
*   History:  
*h*
**********************************************************************/

#include "nsm.h"
#include "Space.h"


device_method_t dp83820_methods[] = {
	/* Device interface */
	DEVMETHOD(device_probe,		dp83820_probe),
	    DEVMETHOD(device_attach,	dp83820_attach),
	    DEVMETHOD(device_detach,	dp83820_detach),
	    DEVMETHOD(device_shutdown,	dp83820_shutdown),
	    DEVMETHOD(device_suspend,	dp83820_suspend),
	    DEVMETHOD(device_resume,	dp83820_resume),

	    {
		0, 0 		}
};

driver_t dp83820_driver = {
	"dp83820",
	    dp83820_methods,
	    sizeof(NsmContext),
};
devclass_t dp83820_devclass;

DRIVER_MODULE(dp83820, pci, dp83820_driver, dp83820_devclass, 0, 0);

/*********************************************************************
 * Name : dp83820_probe 
 *
 * Description :
 *
 *	dp83820_Probe routine checks for the device Id and the 
 *	vendor Id of DP83820 card and returns the device name if 
 *	it matches.
 *
 * Arguments :
 *
 *	dev  This is a device_t structure.
 *
 * Return Values
 *
 *	0      On  Success
 *	ENXIO  On  Failure
 *
 *********************************************************************/

INT
dp83820_probe(device_t dev)
{
	/* Check for the vendor Id of the card */
	if (pci_get_vendor(dev) == VENDOR_ID) {
		device_set_desc(dev, "National Semiconductor DP83820 Gigabit Ethernet Adapter \n Driver ver 1.2");
		return 0;
	}
	return ENXIO;
}

/*********************************************************************
 * Name : dp83820_attach
 *
 * Description :
 *
 *	dp83820_attach  does the following :
 *
 *	* Mapping adapters Operational Register (Memory I/O or Port I/O).
 *	* Initialize interrupt handler.
 *	* Initialize the driver entry points in ifnet structure and attach 
 *	  the ifnet structure.
 *	* Initialize the interface media structure and provide call back 
 *	  functions for getting media status and setting media.
 *	* Intiialize the transmit descriptors, receive descriptors and 
 *	  receive buffers.
 *	* Attach the interface to OS using ether_attach.
 *
 *
 * Arguments :
 *
 *	dev  This is a device_t structure.
 *
 * Return Values
 *
 *	0      On  Success
 *	ENXIO  On  Failure
 *
 *********************************************************************/

INT
dp83820_attach(device_t dev)
{
	NsmContext *pNsm;
	AdapterContext *pAdapter;
	struct ifnet *ifp;
	INT s;
	INT rid,i;
	UINT32 command;
	const INT *media;
	INT nmedia;
	INT error = 0,err;
	ULONG CFLTval;
	UINT Srr_val,Cfg_val;

	s = splimp();

	pNsm = (NsmContext *) device_get_softc(dev);

	bzero(pNsm,sizeof(NsmContext));

	pAdapter = &pNsm->Adapter;
	pAdapter->pNsmContext = (PVOID) pNsm;

	callout_handle_init(&pNsm->tbi_led_handle);

	command = pci_read_config(dev, PCIR_COMMAND, 4);
	if (!(command & PCIM_CMD_BUSMASTEREN)) {
		printf("\n Bus Master not enabled");
		error = ENXIO;
		goto fail;
	}

	if (command & PCIM_CMD_MEMEN) {
		rid = CFGMA;
		pNsm->mapping = SYS_RES_MEMORY;
	} else if (command & PCIM_CMD_PORTEN) {
		rid = CFGIOA;
		pNsm->mapping = SYS_RES_IOPORT;
	} else{
		printf("\n No mapping ");
		error = ENXIO;
		goto fail;
	}

	pNsm->regmap = bus_alloc_resource(dev,pNsm->mapping, &rid,
	    0, ~0, 1, RF_ACTIVE);
	if (pNsm->regmap == NULL) {
		printf("\n No resource for Memory or Port Map");
		error = ENXIO;
		goto fail;
	}
	pNsm->bus_tag = rman_get_bustag(pNsm->regmap);
	pNsm->bus_handle = rman_get_bushandle(pNsm->regmap);


/*	Work Around for a 8 byte Replication issue.
 *	This is typically required for 32 bit bus and if the
 *	silicon revision of the card is less  than or equal to 0x103
 *	Fix: Increase the latency timer( ie bits 8 to 15 of CFGLAT)
 *	value to 0xFE
 */

	NsmRegRead32(pAdapter->pNsmContext, SRR, &Srr_val);
	if(Srr_val <= 0x103)
	{
		NsmRegRead32(pAdapter->pNsmContext, CFG, &Cfg_val);
		if(!(Cfg_val & PCI64_DET))
		{
			CFLTval = pci_read_config(dev, CFGLAT, 4);
			CFLTval &= 0xFFFF00FF;
			CFLTval |= 0xFE00;
			pci_write_config(dev, CFGLAT, CFLTval, 4);
		}
	}

	CFLTval = pci_read_config(dev, CFGLAT, 4);
	pNsm->CacheLineSize= 4*((UCHAR) CFLTval & 0xFF);


	if( AdapterInitialise(pNsm, RX_BUF_SIZE) != SUCCESS) {
		NsmFreeResources(dev,pNsm,MEMORY_IOPORT_FAILURE);
		error = ENXIO;
		goto fail;
	}

	/* Allocate our interrupt.  */
	rid = 0;
	pNsm->irq = bus_alloc_resource(dev, SYS_RES_IRQ, &rid, 0, ~0, 1,
	    RF_SHAREABLE | RF_ACTIVE);
	if (pNsm->irq == NULL) {
		printf("\n No resource for IRQ");
		NsmFreeResources(dev,pNsm,HSMINIT_FAILURE);
		error = ENXIO;
		goto fail;
	}


	err = bus_setup_intr(dev, pNsm->irq, INTR_TYPE_NET,
	    dp83820_intr,pNsm, &pNsm->Irqhandle);
	if (err) {
		printf("\n Bus setup intr failed");
		NsmFreeResources(dev,pNsm,INTERRUPT_RESOURCE_FAILURE);
		error = ENXIO;
		goto fail;
	}

	/* Register the driver entry points in the ifnet structure and 
		 * report it to the OS by ether_ifattach
		 */
	HsmGetMacAddress(pAdapter);
	bcopy(pAdapter->PermMacAddr,(char *)&pNsm->arpcom.ac_enaddr,ETHER_ADDR_LEN);

	/* Intializing the media to the interface */

	ifmedia_init(&pNsm->dp83820_media,0,dp83820_ifmedia_update,
	    dp83820_ifmedia_status);
	/*Add all the supported media to the interface */

	media = dp83820_media_standard;
	nmedia = sizeof(dp83820_media_standard)/sizeof(dp83820_media_standard[0]);
	for (i=0;i<nmedia;i++) {
		ifmedia_add(&pNsm->dp83820_media,media[i],0,NULL);
	}

	ifmedia_set(&pNsm->dp83820_media,DP83820_MEDIA_STANDARD_DEFMEDIA);

	/* End of Initialization to the interface */

	pNsm->unit = device_get_unit(dev);
	ifp = &pNsm->arpcom.ac_if;
	ifp->if_unit = device_get_unit(dev);
	ifp->if_name = "dp";
	ifp->if_output = ether_output;
	ifp->if_baudrate = 1000000000;
	ifp->if_init = dp83820_init;
	ifp->if_softc = pNsm;
	ifp->if_flags = IFF_BROADCAST | IFF_SIMPLEX | IFF_MULTICAST;
	ifp->if_ioctl = dp83820_ioctl;
	ifp->if_start = dp83820_send;
	ifp->if_watchdog = dp83820_watchdog;
	ifp->if_hwassist = 0;

#ifdef TXCHECKSUM
	ifp->if_hwassist = CHECKSUM_FEATURES;
#endif
	/* Attach the inerface */
	ether_ifattach(ifp, ETHER_BPF_SUPPORTED);
	/*
			 * Let the system queue as many packets as we have available
			 * TX depNsmriptors.
			 */
	ifp->if_snd.ifq_maxlen =  MAX_PKTS;

fail:
	splx(s);
	return error;
}

/*********************************************************************
 * Name : dp83820_init
 *
 * Description :
 *
 *	This entry point is called by OS to initialize the adapter. 
 *	This function is typically called when end-user tries to configure
 *	interface up using ifconfig.
 *	This function makes the adapter to start with clean state. 
 *	Indicate to the OS that  interface is running and active.
 *
 * Arguments :
 *
 *	pv  void pointer which points to NsmContext
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID 
dp83820_init(void *pv)
{
	NsmContext	*pNsm = pv;
	AdapterContext	*pAdapter;
	INT s,ret_val;
	struct ifnet	*ifp = &pNsm->arpcom.ac_if;
	INT error;

	if (ifp->if_flags & IFF_RUNNING)
		return;

	s = splimp();

	pAdapter = &(pNsm->Adapter);

	bcopy(&pNsm->arpcom.ac_enaddr,pAdapter->CurrMacAddr,ETHER_ADDR_LEN);

	ret_val = HsmReset(pAdapter, 1);

	if ( ret_val == FAILURE ) {
		error = ENXIO;
		goto fail;
	}
	ret_val = HsmOpen(pAdapter,1);
	if ( ret_val == FAILURE ) {
		error = ENXIO;
		goto fail;
	}

	pNsm->rxFlags = ACCEPT_PERFECT_MATCH_ON | ACCEPT_ALL_BROADCAST_ON;
	HsmRxFilter(pAdapter,pNsm->rxFlags);

	ifp->if_flags |= IFF_RUNNING;
	ifp->if_flags &= ~IFF_OACTIVE;

fail :
	splx(s);
	return;
}
/*********************************************************************
 * Name : dp83820_detach
 *
 * Description :
 *
 *	This entry point is called by the OS to detach the adapter.
 *	Here we free all the resources that were allocated in the 
 *	attach and in the dp83820_init routine.
 *
 * Arguments :
 *
 *	dev  This is a device_t structure.
 *
 * Return Values
 *
 *	None
 *
 *********************************************************************/

INT
dp83820_detach(device_t dev)
{

	INT s;

	s = splimp();

	FreeResources(dev);

	splx(s);

	return 0;
}


INT
dp83820_resume(device_t dev)
{
	return 0;
}

/*********************************************************************
 * Name : dp83820_shutdown
 *
 * Description :
 *
 *	This entry point is called by the OS during shutdown the system.
 *	Here we free all the resources that were allocated in the 
 *	dp83820_attach and in the dp83820_init routine.
 *
 * Arguments :
 *
 *	dev  This is a device_t structure.
 *
 * Return Values
 *
 *	None
 *
 *********************************************************************/

VOID
dp83820_shutdown(device_t dev)
{
	NsmContext *pNsm;
	pNsm = device_get_softc(dev);
	FreeResources(dev);
	return ;
}

INT
dp83820_suspend(device_t dev)
{
	return 0;
}

/*********************************************************************
 * Name : dp83820_watchdog
 *
 * Description :
 *
 *	Watchdog is a transmit timeout handler. Called when a  
 *	transmission is started on the interface, but no tx completed 
 *	interrupt is  received before the timeout. This usually indicates 
 *	that the   card has wedged for some reason.
 *	So we stop the interface,reset the card and call the init 
 *	routine  to reinitialise all the tx/rx lists. After this we check
 *	for the send queue if it is not null then the start routine is 
 *	called.
 *
 * Arguments :
 *
 *	ifp This is a pointer to the ifnet structure
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID 
dp83820_watchdog(struct ifnet *ifp)
{
	NsmContext *pNsm;

	pNsm = (NsmContext *)ifp->if_softc;
#ifdef NSMDEBUG
	printf("\n Watchdog Called !");
#endif
	ifp->if_oerrors ++;
	dp83820_stop(pNsm);
	dp83820_init(pNsm);

	return ;
}

/*********************************************************************
 * Name : dp83820_stop
 *
 * Description :
 *
 *	Stop the driver and free any mbufs allocated to the RX and TX 
 *	lists.
 *
 * Arguments :
 *
 *	pNsm Pointer to NsmContext
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
dp83820_stop(NsmContext *pNsm)
{
	struct ifnet		*ifp;
	AdapterContext *pAdapter;

	ifp = &pNsm->arpcom.ac_if;

	ifp->if_timer = 0;

	pAdapter = &(pNsm->Adapter);

	HsmClose(pAdapter);

	ifp->if_flags &= ~(IFF_RUNNING | IFF_OACTIVE);

	return ;
}


/*********************************************************************
 * Name : AdapterInitialise 
 *
 * Description :
 *
 *  	Here we  initialise the AdapterContext structure variables  
 *
 * Arguments :
 *
 *	pNsm Pointer to NsmContext
 *
 * Return Values
 *
 * 	SUCCESS - On Success
 * 	FAILURE - On Failure
 *
 *********************************************************************/

UCHAR
AdapterInitialise(NsmContext *pNsm,UINT PktSize)
{
	AdapterContext *pAdapter;
	UCHAR i=0;

	pAdapter = &pNsm->Adapter;

/*
 *	RegAddr is ZERO since freebsd used bus tag and
 *	bus handle
 */

	pAdapter->RegAddr = 0;
	pAdapter->UserMediaSpeed = DEFAULT_SPEED;
	pAdapter->UserDuplexMode = FULL_DUPLEX;
	pAdapter->OsPrio = OS_PRIO;

	pAdapter->TxMaxDMASize = pAdapter->RxMaxDMASize = (USHORT) (4 * pNsm->CacheLineSize);

	for(i=0; i< MAXPRI_ADAPTER; i++)
	{
		pAdapter->TxQueueSz[i] = TRANSMIT_DESC_COUNT;
		pAdapter->RxQueueSz[i] = RECEIVE_DESC_COUNT;
	}

	if(HsmInitContext(pAdapter) != SUCCESS)
	{
		printf("HsmInitContext Failed \n");
		return FAILURE;
	}
	

/*
 * 	Indicate that OS doesn't support priority queuing,
 * 	wake on LAN features,Pause frames,Checksum offload and vlan.
 */

	pAdapter->NcBit = DEFAULT_NCBIT;
	pAdapter->PhysCapabilities &= ~VLAN_TAG_INSERTION_PP_ON;
	pAdapter->PhysCapabilities &= ~VLAN_TAG_INSERTION_GEN_ON;
	pAdapter->PhysCapabilities &= ~TX_CHKSUM_OFFLOAD_ON_GEN;
	pAdapter->PhysCapabilities &= ~TX_CHKSUM_OFFLOAD_ON_PP;
	pAdapter->PhysCapabilities &= ~SET_RX_CHKSUM_OFFLOAD_ON;
	pAdapter->PhysCapabilities &= ~SET_WOL_ON;
	pAdapter->PhysCapabilities &= ~SET_BIG_ENDIAN_MODE;
	pAdapter->FlowControl &= ~(TX_PAUSE_FRAMES_ENABLE | RX_PAUSE_FRAMES_ENABLE);
 	/* Enable Auto neg on and Jumbo Frame on */
	pAdapter->PhysCapabilities |= AUTO_NEG_ON;
	pAdapter->PhysCapabilities |= JUMBO_FRAMES_ON;
	pAdapter->MaxPktSize = PktSize;


#ifdef TXCHECKSUM
 	/* Enable Transmit Checksum offload on */
	pAdapter->PhysCapabilities |= TX_CHKSUM_OFFLOAD_ON_PP;
#endif

#ifdef RXCHECKSUM
 	/* Enable Receive Checksum offload on */
	pAdapter->PhysCapabilities |= SET_RX_CHKSUM_OFFLOAD_ON;
#endif
	if(VlanInsertionMode == VLAN_INSERTION_NIC) 
	{
/*
 *		To Enable insertion and extraction of vlan tag
 *		in the NIC
 */
		pAdapter->PhysCapabilities |=  (VLAN_TAG_REMOVAL_ON | MULTIPLE_VLAN_TAG_ON);
	}
	else if(VlanInsertionMode == VLAN_INSERTION_OS) 
	{
/*
 *		To Enable accepting of vlan packets when using
 *		OS supported encapsulation
 */
		pAdapter->PhysCapabilities &=  ~DISCARD_VLAN_TAGGED_PACKETS;
	}
	else
	{
/*
 *		Reject vlan tagged Packets
 *		in the NIC
 */
		pAdapter->PhysCapabilities |=  DISCARD_VLAN_TAGGED_PACKETS;
	}

	if ( HsmInitialize(pAdapter) != SUCCESS ) 
	{
		printf(" Hsminitialise Failed \n");
		return FAILURE;
	}
	return SUCCESS;
}
