
/********************************************************************
*h*
*   Name:	
*
*   		nsm.c
*
*   Description :	
*
*   		This file contains nsm related  functions.
*  
* 
*   History:  
*h*
**********************************************************************/

#include <nsm.h>
#include <hsm.h>

/*********************************************************************
 * Name : NsmMalloc 
 *
 * Description :
 *
 *	This function is called to allocate memory.
 *
 * Arguments :
 *
 *	size	The size of the memory that needs to be allocated.
 *
 * Return Values
 *
 *	NULL  On Failure.
 *	PVOID On Success.
 *
 *********************************************************************/

PVOID 
NsmMalloc(UINT size)
{
	PVOID Addr;
	Addr = malloc(size, M_DEVBUF, M_NOWAIT);
	if (Addr != NULL)
		bzero(Addr,size);
	return(Addr);
}

/*********************************************************************
 * Name : Nsmsleep
 *
 * Description :
 *
 *	This routine is called by hsm to sleep for the specified 
 *	amount of time.
 *
 * Arguments :
 *
 *	pNsm       pointer to the NsmContext structure.
 *	MilliSecs  Sleep time in milliseconds.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID 	
NsmSleep(NsmContext *pNsmContext, int MilliSecs)
{
	DELAY(MilliSecs*1000);

}

/*********************************************************************
 * Name : dp83820_mbuffree
 *
 * Description :
 *
 *	This is called by the Os when the mbuf has an  external buffer 
 *	to be freed.
 *	Since the buffer associated with the paricular mbuf is with the
 *	driver until it is unloaded hence when we do not free the buffer
 *	here. We just return.
 *
 * Arguments :
 *
 *	buf    pointer  to the buffer which needs to be freed
 *	size   size of the buffer to be freed
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
dp83820_mbuffree(caddr_t buf,UINT size)
{
	return ;
}

/*********************************************************************
 * Name : dp83820_mbufref
 *
 * Description :
 *
 *	This is called by the Os when the mbufs  buffer has an external
 *	reference object to be manipulated.
 *	Since we are handling that in NsmRxPackets we just return from 
 *	this function.
 *
 * Arguments :
 *
 *	buf    pointer  to the buffer which needs to be freed
 *	size   size of the buffer to be freed
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
dp83820_mbufref(caddr_t buf,UINT size)
{
	printf(" MBUFref called ");
	return ;
}

/*********************************************************************
 * Name : NsmMallocContiguous
 *
 * Description :
 *
 *	This is called to allocate a contiguous chunk of memory, the 
 *	virtual & physical addresses of the start of the chunk of memory,
 *	is set in the AddrStruct structure. The bit alignment required is 
 *	given by the Alignment parameter.
 *
 * Arguments :
 *
 *	pNsm       Pointer to the NsmContext structure.
 *	pAddr      Pointer to the AddrStruct structure, where the virtual
 *	           and physical addresses need to be stored.
 *	alignment  The alignment factor, to which the memory allocated 
 *                 needs to be aligned .
 *	cached     Unused.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
NsmMallocContiguous(NsmContext *pNsm, AddrStruct *pAddr, UINT alignment, UINT cached)
{

	CHAR  *v_mem = NULL;
	UINT  align;

	align = (UINT)alignment/8;

	/* Allocate physically contiguous memory of size 
		 * (pAddrStruct->BufSize) 
	 	 *  align parameter to take care of alignment issues.
		 */

	v_mem = contigmalloc(pAddr->BufSize, M_DEVBUF,M_NOWAIT, 0, 0xffffffff,align,0);
	if(v_mem == NULL)
	{
		pAddr->VirtAddr = NULL;
		pAddr->PhysAddr = NULL;
		return;
	}

	bzero(v_mem,(pAddr->BufSize));

	pAddr->pCookie = v_mem;

	pAddr->VirtAddr = (UCHAR *)v_mem;
	pAddr->PhysAddr = (UCHAR *)vtophys(v_mem);

	return;
}

/*********************************************************************
 * Name : NsmFreeContiguous
 *
 * Description :
 *
 *	This is called by HSM to free a contiguous chunk of memory.
 *
 * Arguments :
 *
 *	pNsm     Pointer to the NsmContext structure.
 *	pAddr    Pointer to the AddrStruct structure, where the virtual
 *	         and physical addresses are stored.
 *	cached   Unused.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID 
NsmFreeContiguous(NsmContext *pNsm,AddrStruct *pAddr, UINT cached)
{
	contigfree(pAddr->VirtAddr,pAddr->BufSize,M_DEVBUF);
	pAddr->pCookie = pAddr->PhysAddr = pAddr->VirtAddr = 0;

}

/*********************************************************************
 * Name : NsmAllocatePacket
 *
 * Description :
 *
 *	This is called by HSM to allocate an mbuf with a data buffer
 *	associated with it.
 *
 * Arguments :
 *
 *	pNsm     Pointer to the NsmContext structure.
 *	pRcb     Pointer to the HsmRCB, to which the allocated packet
 *	         is to be linked to.
 *	pAddr    Pointer to the AddrStruct structure, where the virtual
 *	         and physical addresses are stored.
 *	cached   Unused.
 *
 * Return Values
 *
 *	SUCCESS  On Success
 *	FAILURE  On Failure
 *
 *********************************************************************/

UCHAR 
NsmAllocatePacket(NsmContext *pNsm, HsmRCB *pRcb, AddrStruct *pAddr)
{
	struct mbuf		*m_new = NULL;

	MGETHDR(m_new, M_DONTWAIT, MT_DATA);
	if (m_new == NULL) {
		return(FAILURE);
	}


	NsmMallocContiguous(pNsm, pAddr,64,0);
	if(pAddr->VirtAddr == NULL)
		return(FAILURE);


	/* Attach the buffer to the mbuf 
		*/

	m_new->m_data = m_new->m_ext.ext_buf = (void *)pAddr->VirtAddr;
	m_new->m_flags |= M_EXT;


	m_new->m_ext.ext_size = pAddr->BufSize;
	m_new->m_pkthdr.len = m_new->m_len = 0;

	m_new->m_ext.ext_free = dp83820_mbuffree;
	m_new->m_ext.ext_ref = dp83820_mbufref;

	pAddr->pCookie =(PVOID) m_new;
	return(SUCCESS);
}

/*********************************************************************
 * Name : NsmDeAllocatePacket
 *
 * Description :
 *
 *	This function is called by the HSM to free the mbuf structure
 *	associated with the HsmRCB structure and  also frees the data
 *	buffer associated with the mbuf structure.
 *
 * Arguments :
 *
 *	pNsm	Pointer to the NsmContext structure
 *	pAddr	Pointer to the AddrStruct structure.
 *	pRcb	Pointer to the  HsmRCB, to which the allocated packet 
 *	        is to be freed.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
NsmDeAllocatePacket(NsmContext *pNsmContext, AddrStruct *pAddr, HsmRCB *pRCB)
{
	struct mbuf		*m_new = NULL;

	m_new = (struct mbuf *)pRCB->pOsCtrlBlk;

	/*
		 * Free the message block 
		 */
	m_freem(m_new);

	/* Free the data block assigned to the message block 
	         */

	NsmFreeContiguous(pNsmContext,pAddr,0);
	pRCB->pOsCtrlBlk = NULL;
	pRCB->pDataBuf = NULL;

}

/*********************************************************************
 * Name : NsmCompare
 *
 * Description :
 *
 *	This routine is used  to compare two strings str1 and str2 
 *	if they are equal we return zero else we return 1.
 *
 * Arguments :
 *
 *	addr1	Pointer to void
 *	addr2 	Pointer to void
 *	Len 	Length in bytes to be compared
 *
 * Return Values
 *
 *	O - SUCEESS
 *	1 - FAILURE
 *
 *********************************************************************/

CHAR
NsmCompare(PVOID addr1, PVOID addr2, UINT len)
{
	UINT i;
	CHAR *tempstr1, *tempstr2;

	tempstr1 = (CHAR *)addr1;
	tempstr2 = (CHAR *)addr2;

	for (i = 0; i < len; i++)
	{
		if(tempstr1[i] != tempstr2[i])
			return 1;
	}
	return 0;
}

/*********************************************************************
 * Name : FreeResources
 *
 * Description :
 *
 * This function is called by detach and shutdown to free all the 
 * resources allocated previously.
 *	
 * Arguments :
 *
 *	dev	 This is a device_t structure
 *	
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/


VOID
FreeResources(device_t dev)
{
	NsmContext *pNsm;
	AdapterContext *pAdapter;

	pNsm =(NsmContext *) device_get_softc(dev);
	pAdapter = &pNsm->Adapter;

	dp83820_stop(pNsm);

	do
	{
		NsmSleep(pNsm,10);
	}while (pAdapter->interruptStatus);

	if(pNsm->tbi_activity)
 		untimeout(LedTimerFunc,(VOID *)pAdapter,pNsm->tbi_led_handle);

	NsmFreeResources(dev,pNsm,DETACH);
}

/*********************************************************************
 * Name : NsmFreeResources
 *
 * Description :
 *
 *	This routine is called to deallocate the resources based
 *	on the failure conditions.
 *
 * Arguments :
 *	dev  This is a device_t structure
 *	pNsm pointer to the NsmContext structure.
 *	cmd  Contains the type of failure.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/


VOID
NsmFreeResources(device_t dev,NsmContext *pNsm,int cmd)
{

	switch(cmd)
	{
	case DETACH:
		ether_ifdetach(&pNsm->arpcom.ac_if, ETHER_BPF_SUPPORTED);
		ifmedia_removeall(&pNsm->dp83820_media);
	case SETUP_INTR_FAILURE :
		bus_teardown_intr(dev, pNsm->irq, pNsm->Irqhandle);
	case INTERRUPT_RESOURCE_FAILURE:
		bus_release_resource(dev, SYS_RES_IRQ, 0, pNsm->irq);
	case HSMINIT_FAILURE :
		if(!pNsm->AdapterUnInitialise)
			HsmUnInitialize(&(pNsm->Adapter));
	case MEMORY_IOPORT_FAILURE:
		if (pNsm->mapping & SYS_RES_MEMORY)
			bus_release_resource(dev, pNsm->mapping, CFGMA, pNsm->regmap);
			else
			bus_release_resource(dev, pNsm->mapping, CFGIOA, pNsm->regmap);
		break;
	}
}
/*********************************************************************
 * Name : NsmStartLedWithTimer
 *
 * Description :
 *
 *	This routine is called to turn the activity LED on in Tbi
 *      for the specified amount of time by calling a timer function
 *
 * Arguments :
 *	pNsm pointer to the NsmContext structure.
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/
VOID
NsmStartLedWithTimer(NsmContext *pNsm)
{
	if(!pNsm->tbi_activity)
	{
		/* Turn ON Activity LED */
		HsmTbiActivityLedOn(&pNsm->Adapter);
		pNsm->tbi_led_handle = timeout(LedTimerFunc, (VOID *) &pNsm->Adapter,
										LED_TIMER_DELAY * hz);
		pNsm->tbi_activity = 1;
	}
}
/*********************************************************************
 * Name : LedTimerFunc
 *
 * Description :
 *
 *	This routine is called when a timer to turn the tbi led on 
 *	expires.So the activity LED in Tbi turned off.
 *
 * Arguments :
 *            arg This is a void pointer to Adapter context 
 *
 * Return Values
 *
 *	None.
 *
 *********************************************************************/

VOID
LedTimerFunc(PVOID arg)
{
	AdapterContext *pAdapter = (AdapterContext *)arg;
	NsmContext     *pNsm     = pAdapter->pNsmContext;

	/* Turn OFF Active LED */
	HsmTbiActivityLedOff(pAdapter);
	pNsm->tbi_activity = 0;
}

/******************************************************************
 * Name         : NsmGetPkt                                       *
 *                                                                *
 * Description  : This routine will get the pOsCtrlBlk and pass   *
 *                the following to the caller.                    *
 *                1. Array of packet data.                        *
 *                2. TCP Checksum virtual addresses.              *
 *                3. Packet length (This is the actual packet     *
 *                   length from the IP header).                  *
 *                                                                *
 * Parameters   : pNsmContext - pointer to the NsmContext.        *
 *                pOsCtrlBlk - pointer to the OsCtrlBlk.          *
 *                pPkt - pointer to the array of packet data.     *
 *                pProtocolChecksumoffset - array of pointers to  *
 *                the virtual address of the checksum field.      *
 *                pPktLength - pointer to the packet length in the*
 *                IP header.                                      *
 *                Protocol - Indicates what type of protocol 	   *
 *                (TCP/UDP) for calculating the checksum offset.  *
 *                                                                *
 * Return Value : None.                                           *
 *                                                                *
 ******************************************************************/

#if defined (TXCHECKSUM)
#if defined (UDPCHECKSUM_HOTFIX) || defined (TCPCHECKSUM_HOTFIX)
VOID NsmGetPkt( NsmContext *pNsmContext, PVOID pOsCtrlBlk,
				UCHAR *pPkt, UCHAR *pProtocolChecksumoffset[],
				USHORT *pPktLength, UINT32 Protocol,
				UINT32 *pNumberOfDescriptors)
{
	struct mbuf *mb;
	UCHAR  *VirtualAddress;
	UINT32 pktIndex = 0, FragLength;
	UINT16 ipHeaderLen = 0;
	UCHAR chkSumFlag = 1;
	UINT32 ProtocolOffset;

	mb = (struct mbuf *)pOsCtrlBlk;
	*pNumberOfDescriptors = 0;

	for ( ; mb != NULL; mb = mb->m_next)
	{
		if (mb->m_len == 0)
			continue;
		else
		{
			VirtualAddress = (UCHAR *)mtod(mb,vm_offset_t);
			FragLength = mb->m_len;
			(*pNumberOfDescriptors) ++;

			if((pktIndex <= IP_HEADERLEN_OFFSET) &&
				((pktIndex + FragLength) > IP_HEADERLEN_OFFSET))
			{
				ipHeaderLen =
					IP_HLEN((VirtualAddress[IP_HEADERLEN_OFFSET - pktIndex]));

				chkSumFlag = 0;
				if(Protocol & TCPPKT)
				{
					ProtocolOffset = TCP_CHECKSUM_OFFSET + ipHeaderLen + ETHER_HEADER_LENGTH;
				}
				else if (Protocol & UDPPKT)
				{
					ProtocolOffset = UDP_CHECKSUM_OFFSET + ipHeaderLen + ETHER_HEADER_LENGTH;
				}
			}

			if(chkSumFlag == 0)
			{
				if((pktIndex <= ProtocolOffset) &&
					((pktIndex + FragLength) > ProtocolOffset))
				{
					pProtocolChecksumoffset[0] =
					(VirtualAddress + (ProtocolOffset - pktIndex));
					ProtocolOffset++;
				}

				if((pktIndex <= ProtocolOffset) &&
					((pktIndex + FragLength) > ProtocolOffset))
				{
					pProtocolChecksumoffset[1] =
					(VirtualAddress + (ProtocolOffset - pktIndex));
					chkSumFlag = 1;
				}
			}
			NsmCopy((PVOID)VirtualAddress, (PVOID)(pPkt + pktIndex), FragLength);
			pktIndex += FragLength;
		}
	}
		*pPktLength =
			(pPkt[IP_LENGTH_OFFSET] << 8) | (pPkt[IP_LENGTH_OFFSET + 1]);
}
#endif
#endif

